import { Box, Step, StepLabel, Stepper, Typography } from '@material-ui/core'
import React, { useContext, useEffect, useState } from 'react'
import Button from '../../button'
import TelegramInstructions from '../tg-connect-instructions/TelegramInstructions'
import Styles from './TelegramWrapper.module.css'
import { createTheme } from '@material-ui/core'
import { ThemeProvider } from '@material-ui/styles'
import TelegramConnect from '../telegram-connect/TelegramConnect'
import { FetchContext } from '../../../store/fetch'
import { Alert, AlertTitle } from '@material-ui/lab'
import { AuthContext } from '../../../store/auth'
import TelegramDisconnect from '../telegram-disconnect/TelegramDisconnect'
import NotificationSnack from '../../notification-snack/NotificationSnack'

const theme = createTheme({
  overrides: {
    MuiStepIcon: {
      root: {
        '&$completed': {
          color: 'green'
        },
        '&$active': {
          color: 'var(--blue-500)'
        }
      },
      active: {},
      completed: {}
    }
  }
})
const steps = ['Instructions', 'Verification', 'Success']

const TelegramWrapper = () => {
  const [activeStep, setActiveStep] = React.useState(0)
  const { authAxios } = useContext(FetchContext)
  const [conenctError, setConnectError] = useState('')
  const { authState, setAuthState } = useContext(AuthContext)
  const [snackbar, setSnackbar] = useState({
    show: false,
    message: '',
    severity: 'success'
  })
  const username = authState.userInfo?.username
  const isTelegramConnected = authState.userInfo?.isTelegramConnected

  useEffect(() => {
    if (isTelegramConnected) {
      setActiveStep(2)
    }
  }, [isTelegramConnected])

  const checkConnection = async () => {
    try {
      const response = await authAxios.post(
        '/user/verify-telegram-connection',
        { username: username }
      )
      if (response.status === 200) {
        setActiveStep((prevActiveStep) => prevActiveStep + 1)
        if (!authState.userInfo.isTelegramConnected) {
          authState.userInfo.isTelegramConnected = true
          setAuthState(authState)
        }
      }
    } catch (error) {
      setConnectError('Not verified yet.')
    }
  }
  const handleNext = async () => {
    if (activeStep === 1) {
      if (!isTelegramConnected) {
        await checkConnection()
      } else {
        setActiveStep((prevActiveStep) => prevActiveStep + 1)
      }
    } else {
      if (activeStep === steps.length - 1) {
        window.location.replace('/users/' + username)
      } else {
        setActiveStep((prevActiveStep) => prevActiveStep + 1)
      }
    }
  }
  const handleBack = () => {
    setConnectError('')
    setActiveStep((prevActiveStep) => prevActiveStep - 1)
  }
  const getStepContent = () => {
    switch (activeStep) {
      case 0:
        return <TelegramInstructions />
      case 1:
        return <TelegramConnect />
      case 2:
        return (
          <TelegramDisconnect
            username={username}
            setActiveStep={setActiveStep}
            setSnackbar={setSnackbar}
          />
        )
      default:
        return 'Unknown step'
    }
  }
  return (
    <>
      <ThemeProvider theme={theme}>
        <Stepper activeStep={activeStep} className={Styles.stepper}>
          {steps.map((step) => {
            return (
              <Step key={step}>
                <StepLabel>
                  <Typography variant="h6">{step}</Typography>
                </StepLabel>
              </Step>
            )
          })}
        </Stepper>
      </ThemeProvider>

      <div className={Styles.telegram_wrapper}>
        {getStepContent()}
        {!!conenctError && (
          <Alert severity="error">
            <AlertTitle>Error</AlertTitle>
            {conenctError}
            <strong>Please try again</strong>
          </Alert>
        )}
        <Box sx={{ display: 'flex', flexDirection: 'row', pt: 2 }}>
          <Button
            secondary
            color="inherit"
            disabled={activeStep === 0}
            onClick={handleBack}
            sx={{ mr: 1 }}
          >
            Back
          </Button>
          <Box sx={{ flex: '1 1 auto' }} />
          <Button onClick={handleNext} primary style={{ padding: 'auto 30px' }}>
            {activeStep === steps.length - 1
              ? 'Go home'
              : activeStep === 1
              ? isTelegramConnected
                ? 'Next'
                : 'Verify'
              : 'Next'}
          </Button>
        </Box>
        <NotificationSnack
          message={snackbar.message}
          showSnackbar={snackbar.show}
          severity={snackbar.severity}
          setShowSnackbar={(show) => setSnackbar({ ...snackbar, show })}
        />
      </div>
    </>
  )
}

export default TelegramWrapper
